<?php

namespace GamesShop\Routing\Api\DataTables;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Criteria;
use Doctrine\DBAL\Query\Expression\ExpressionBuilder;
use Doctrine\ORM\EntityManager;
use GamesShop\ContainerHandler;
use GamesShop\Entities\Games\Key;
use GamesShop\Entities\Games\KeyState;
use GamesShop\Entities\GamesList;
use GamesShop\Login\LoginHandler;
use GamesShop\Login\UserPermission;
use GamesShop\UserManager;
use Laminas\Diactoros\Response\JsonResponse;
use League\Route\Http\Exception\ForbiddenException;
use League\Route\Http\Exception\UnauthorizedException;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

class AvailableKeysEndpoint
{
    public function __construct(
        private readonly LoginHandler $loginHandler,
        private readonly EntityManager $entityManager,
    )
    {
    }

    public function __invoke(ServerRequestInterface $request): ResponseInterface
    {
        if (!$this->loginHandler->isLoggedIn()) {
            throw new UnauthorizedException();
        }

        $user = $this->loginHandler->getCurrentUser();
        if (!$user->getPermission()->hasLevel(UserPermission::VIEWER)) {
            throw new ForbiddenException();
        }
        
        $applicableLists = ContainerHandler::get(UserManager::class)
            ->getApplicableGameLists($user);
        
        $keyRepo = $this->entityManager->getRepository(Key::class);
        $keys = $keyRepo->matching(Criteria::create()
            ->where(Criteria::expr()->in('list', $applicableLists->toArray()))
            ->andWhere(Criteria::expr()->eq('state', KeyState::AVAILABLE))
        );
        
        $games = new ArrayCollection();
        foreach ($keys as $key) {
            $game = $key->getGame();
            
            if ($games->contains($game)) {
                continue;
            }
            
            $games->add($game);
        }
        
        return new JsonResponse(
            [
                'data' =>
                    $games
                        ->map(fn ($game) => [
                            'name' => $game->getName(),
                            'id' => $game->getId(),
                        ])
                        ->toArray()
            ]
        );
    }
}