<?php
declare(strict_types=1);

namespace GamesShop\Routing\Api\Web;

use Doctrine\ORM\EntityManager;
use GamesShop\Entities\GamesList;
use GamesShop\Importer\GameImporter;
use GamesShop\Login\LoginHandler;
use GamesShop\Login\UserPermission;
use Laminas\Diactoros\Response\JsonResponse;
use Laminas\Diactoros\UploadedFile;
use League\Route\Http\Exception\BadRequestException;
use League\Route\Http\Exception\ForbiddenException;
use League\Route\Http\Exception\UnauthorizedException;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

final class ImportKeysRoute
{
    public function __construct(
        private readonly LoginHandler $loginHandler,
        private readonly GameImporter $importer,
        private readonly EntityManager $entityManager,
    ) { }

    public function __invoke(ServerRequestInterface $request): ResponseInterface
    {
        if (!$this->loginHandler->isLoggedIn()) {
            throw new UnauthorizedException();
        }

        $user = $this->loginHandler->getCurrentUser();
        if (!$user->getPermission()->hasLevel(UserPermission::PROVIDER)) {
            throw new ForbiddenException();
        }

        $body = $request->getParsedBody();
        if (!array_key_exists('listid', $body)) {
            throw new BadRequestException();
        }

        $list = $this->entityManager->getRepository(GamesList::class)->findOneBy([ 'owner' => $user, 'id' => $body['listid'] ]);
        if (!$list instanceof GamesList) {
            throw new BadRequestException();
        }

        /**
         * @var UploadedFile $file
         */
        $file = $request->getUploadedFiles()['file'] ?? null;
        if (!$file === null) {
            throw new BadRequestException();
        }

        $fileName = tempnam(sys_get_temp_dir(), 'ImportKeys');
        $file->moveTo($fileName);

        $columnDefs = $request->getParsedBody()['columns'];

        [$total, $imported] = $this->importer->import($fileName, $columnDefs, $list);
        unlink($fileName);

        return new JsonResponse([ 'success' => true, 'total' => $total, 'imported' => $imported ]);
    }
}